@@ -7,14 +7,15 @@ L.Map.mergeOptions({
     hash: true,
     embedControl: true,
     datalayersControl: true,
-    default_color: "DarkBlue",
+    default_color: 'DarkBlue',
     default_smoothFactor: 1.0,
     default_opacity: 0.5,
     default_fillOpacity: 0.3,
     default_stroke: true,
     default_fill: true,
     default_weight: 3,
-    default_iconClass: "Default",
+    default_iconClass: 'Default',
+    default_zoomTo: 16,
     attributionControl: true,
     allowEdit: true,
     homeControl: true,
@@ -40,8 +41,8 @@ L.Map.mergeOptions({
         // {url: 'http://localhost:8019/en/datalayer/1502/', label: 'Simplified World Countries', format: 'geojson'}
     ],
     moreControl: true,
-    jumpToZoom: 15,
-    captionBar: false
+    captionBar: false,
+    slideshow: {}
 });
 
 L.Storage.Map.include({
@@ -51,7 +52,7 @@ L.Storage.Map.include({
         if (geojson.properties && geojson.properties.locale) {
             L.setLocale(geojson.properties.locale);
         }
-        var zoomControl = typeof geojson.properties.zoomControl !== "undefined" ? geojson.properties.zoomControl : true;
+        var zoomControl = typeof geojson.properties.zoomControl !== 'undefined' ? geojson.properties.zoomControl : true;
         geojson.properties.zoomControl = false;
         L.Util.setBooleanFromQueryString(geojson.properties, 'scrollWheelZoom');
         L.Map.prototype.initialize.call(this, el, geojson.properties);
@@ -63,14 +64,14 @@ L.Storage.Map.include({
             this.options.center = geojson.geometry;
         }
         this.options.zoomControl = zoomControl;
-        this.overrideBooleanOptionFromQueryString("zoomControl");
-        this.overrideBooleanOptionFromQueryString("moreControl");
-        this.overrideBooleanOptionFromQueryString("miniMap");
-        this.overrideBooleanOptionFromQueryString("scaleControl");
-        this.overrideBooleanOptionFromQueryString("allowEdit");
-        this.overrideBooleanOptionFromQueryString("datalayersControl");
-        this.overrideBooleanOptionFromQueryString("displayDataBrowserOnLoad");
-        this.overrideBooleanOptionFromQueryString("displayCaptionOnLoad");
+        this.overrideBooleanOptionFromQueryString('zoomControl');
+        this.overrideBooleanOptionFromQueryString('moreControl');
+        this.overrideBooleanOptionFromQueryString('miniMap');
+        this.overrideBooleanOptionFromQueryString('scaleControl');
+        this.overrideBooleanOptionFromQueryString('allowEdit');
+        this.overrideBooleanOptionFromQueryString('datalayersControl');
+        this.overrideBooleanOptionFromQueryString('displayDataBrowserOnLoad');
+        this.overrideBooleanOptionFromQueryString('displayCaptionOnLoad');
         if (L.Browser.ielt9) {
             // TODO include ie9
             this.options.allowEdit = false;
@@ -108,28 +109,24 @@ L.Storage.Map.include({
         this.datalayers_index = Array();
         this.deleted_datalayers = Array();
         // create datalayers
-        for(var j in this.options.datalayers) {
-            if(this.options.datalayers.hasOwnProperty(j)){
-                this._createDataLayer(this.options.datalayers[j]);
-            }
-        }
+        this.initDatalayers();
         if (this.options.displayCaptionOnLoad) {
             // Retrocompat
             if (!this.options.onLoadPanel) {
-                this.options.onLoadPanel = "caption";
+                this.options.onLoadPanel = 'caption';
             }
             delete this.options.displayCaptionOnLoad;
         }
         if (this.options.displayDataBrowserOnLoad) {
             // Retrocompat
             if (!this.options.onLoadPanel) {
-                this.options.onLoadPanel = "databrowser";
+                this.options.onLoadPanel = 'databrowser';
             }
             delete this.options.displayDataBrowserOnLoad;
         }
-        if (this.options.onLoadPanel === "databrowser") {
+        if (this.options.onLoadPanel === 'databrowser') {
             this.openBrowser();
-        } else if (this.options.onLoadPanel === "caption") {
+        } else if (this.options.onLoadPanel === 'caption') {
             this.displayCaption();
         }
 
@@ -149,9 +146,9 @@ L.Storage.Map.include({
                         self.fire('isdirty');
                     }
                     if (status) {
-                        L.DomUtil.addClass(this._container, "storage-is-dirty");
+                        L.DomUtil.addClass(this._container, 'storage-is-dirty');
                     } else {
-                        L.DomUtil.removeClass(this._container, "storage-is-dirty");
+                        L.DomUtil.removeClass(this._container, 'storage-is-dirty');
                     }
                     isDirty = status;
                 }
@@ -203,55 +200,47 @@ L.Storage.Map.include({
             this.on('draw:start', function (e) {
                 e.layer.isDirty = true;
             });
-            L.Storage.on('ui:end ui:start', function (e) {
+            L.Storage.on('ui:end ui:start', function () {
                 this.editedFeature = null;
             }, this);
             this.initEditBar();
             var editShortcuts = function (e) {
-                var key = e.keyCode,
-                chars = {
-                    E: 69,
-                    H: 72,
-                    I: 73,
-                    L: 76,
-                    M: 77,
-                    P: 80,
-                    S: 83,
-                    Z: 90
-                };
-                if (key == chars.E && e.ctrlKey && !this.editEnabled) {
+                var key = e.keyCode;
+                if (key == L.S.Keys.E && e.ctrlKey && !this.editEnabled) {
                     L.DomEvent.stop(e);
                     this.enableEdit();
-                } else if (key == chars.E && e.ctrlKey && this.editEnabled && !this.isDirty) {
+                } else if (key == L.S.Keys.E && e.ctrlKey && this.editEnabled && !this.isDirty) {
                     L.DomEvent.stop(e);
                     this.disableEdit();
                     L.S.fire('ui:end');
                 }
-                if (key == chars.S && e.ctrlKey && this.isDirty) {
+                if (key == L.S.Keys.S && e.ctrlKey) {
                     L.DomEvent.stop(e);
-                    this.save();
+                    if (this.isDirty) {
+                        this.save();
+                    }
                 }
-                if (key == chars.Z && e.ctrlKey && this.isDirty) {
+                if (key == L.S.Keys.Z && e.ctrlKey && this.isDirty) {
                     L.DomEvent.stop(e);
                     this.askForReset();
                 }
-                if (key == chars.M && e.ctrlKey && this.editEnabled) {
+                if (key == L.S.Keys.M && e.ctrlKey && this.editEnabled) {
                     L.DomEvent.stop(e);
                     this._controls.draw.startMarker();
                 }
-                if (key == chars.P && e.ctrlKey && this.editEnabled) {
+                if (key == L.S.Keys.P && e.ctrlKey && this.editEnabled) {
                     L.DomEvent.stop(e);
                     this._controls.draw.startPolygon();
                 }
-                if (key == chars.L && e.ctrlKey && this.editEnabled) {
+                if (key == L.S.Keys.L && e.ctrlKey && this.editEnabled) {
                     L.DomEvent.stop(e);
                     this._controls.draw.startPolyline();
                 }
-                if (key == chars.I && e.ctrlKey && this.editEnabled) {
+                if (key == L.S.Keys.I && e.ctrlKey && this.editEnabled) {
                     L.DomEvent.stop(e);
                     this.importPanel();
                 }
-                if (key == chars.H && e.ctrlKey && this.editEnabled) {
+                if (key == L.S.Keys.H && e.ctrlKey && this.editEnabled) {
                     L.DomEvent.stop(e);
                     this.help.show('edit');
                 }
@@ -259,13 +248,14 @@ L.Storage.Map.include({
             L.DomEvent.addListener(document, 'keydown', editShortcuts, this);
         }
 
-        window.onbeforeunload = function (e) {
+        window.onbeforeunload = function () {
             if (isDirty) {
                 return true;
             }
         };
         this.backupOptions();
         this.initContextMenu();
+        this.slideshow = new L.S.Slideshow(this, this.options.slideshow);
     },
 
     overrideBooleanOptionFromQueryString: function (name) {
@@ -279,7 +269,7 @@ L.Storage.Map.include({
             delete this._controls[i];
         }
 
-        L.DomUtil.classIf(document.body, 'storage-caption-bar-enabled', this.options.captionBar);
+        L.DomUtil.classIf(document.body, 'storage-caption-bar-enabled', this.options.captionBar || (this.options.slideshow && this.options.slideshow.delay));
         if (this.options.zoomControl) {
             this._controls.zoomControl = (new L.Control.Zoom({zoomInTitle: L._('Zoom in'), zoomOutTitle: L._('Zoom out')})).addTo(this);
         }
@@ -337,6 +327,40 @@ L.Storage.Map.include({
 
     },
 
+    initDatalayers: function () {
+        var toload = 0, datalayer, seen = 0, self = this;
+        var decrementToLoad = function () {
+            toload--;
+            if (toload === 0) {
+                loaded();
+            }
+        };
+        var loaded = function () {
+            self.fire('datalayersloaded');
+            self.datalayersLoaded = true;
+        };
+        for(var j in this.options.datalayers) {
+            if(this.options.datalayers.hasOwnProperty(j)){
+                toload++;
+                seen++;
+                datalayer = this._createDataLayer(this.options.datalayers[j]);
+                datalayer.onceLoaded(decrementToLoad);
+            }
+        }
+        if (seen === 0) { // no datalayer
+            loaded();
+        }
+    },
+
+    onceDatalayersLoaded: function (callback, context) {
+        if (this.datalayersLoaded) {
+            callback.call(context || this, this);
+        } else {
+            this.once('datalayersloaded', callback, context);
+        }
+        return this;
+    },
+
     updateDatalayersControl: function () {
         if (this._controls.datalayersControl) {
             this._controls.datalayersControl.update();
@@ -459,14 +483,14 @@ L.Storage.Map.include({
     },
 
     getDefaultOption: function (option) {
-        return this.options["default_" + option] || null;
+        return this.options['default_' + option] || null;
     },
 
     updateExtent: function() {
         this.options.center = this.getCenter();
         this.options.zoom = this.getZoom();
         this.isDirty = true;
-        L.Storage.fire("ui:alert", {content: L._('The zoom and center have been setted.'), "level": "info"});
+        L.Storage.fire('ui:alert', {content: L._('The zoom and center have been setted.'), 'level': 'info'});
     },
 
     updateTileLayers: function () {
@@ -511,7 +535,7 @@ L.Storage.Map.include({
             L.DomUtil.create('hr', '', container);
             L.DomUtil.add('h4', '', container, L._('Short URL'));
             var shortUrl = L.DomUtil.create('input', 'storage-short-url', container);
-            shortUrl.type = "text";
+            shortUrl.type = 'text';
             shortUrl.value = this.options.shortUrl;
         }
         L.DomUtil.create('hr', '', container);
@@ -519,7 +543,7 @@ L.Storage.Map.include({
         L.DomUtil.add('p', '', container, L._('Only visible features will be downloaded.'));
         var download = L.DomUtil.create('a', 'button', container);
         download.innerHTML = L._('Download data');
-        download.download = "features.geojson";
+        download.download = 'features.geojson';
         L.DomEvent.on(download, 'click', function () {
             var content = JSON.stringify(this.toGeoJSON(), null, 2);
             window.URL = window.URL || window.webkitURL;
@@ -537,7 +561,7 @@ L.Storage.Map.include({
             }
         });
         var geojson = {
-            type: "FeatureCollection",
+            type: 'FeatureCollection',
             features: features
         };
         return geojson;
@@ -569,18 +593,18 @@ L.Storage.Map.include({
             map = this, option,
             types = ['geojson', 'csv', 'gpx', 'kml', 'osm', 'georss'];
         title.innerHTML = L._('Import data');
-        fileInput.type = "file";
-        submitInput.type = "button";
+        fileInput.type = 'file';
+        submitInput.type = 'button';
         submitInput.value = L._('Import');
-        submitInput.className = "button";
+        submitInput.className = 'button';
         typeLabel.innerHTML = L._('Choose the format of the data to import');
         this.help.button(typeLabel, 'importFormats');
         var typeInput = L.DomUtil.create('select', '', typeLabel);
-        typeInput.name = "format";
+        typeInput.name = 'format';
         layerLabel.innerHTML = L._('Choose the layer to import in');
         var layerInput = L.DomUtil.create('select', '', layerLabel);
-        layerInput.name = "datalayer";
-        urlInput.type = "text";
+        layerInput.name = 'datalayer';
+        urlInput.type = 'text';
         urlInput.placeholder = L._('Provide an URL here');
         rawInput.placeholder = L._('Paste here your data');
         this.eachDataLayer(function (datalayer) {
@@ -710,7 +734,7 @@ L.Storage.Map.include({
                 django: 'https://www.djangoproject.com',
                 umap: 'http://wiki.openstreetmap.org/wiki/UMap'
             };
-        umapCredit.innerHTML = L._("Powered by <a href='{leaflet}'>Leaflet</a> and <a href='{django}'>Django</a>, glued by <a href='{umap}'>uMap project</a>.", urls);
+        umapCredit.innerHTML = L._('Powered by <a href="{leaflet}">Leaflet</a> and <a href="{django}">Django</a>, glued by <a href="{umap}">uMap project</a>.', urls);
 
         L.S.fire('ui:start', {data: {html: container}});
     },
@@ -799,23 +823,26 @@ L.Storage.Map.include({
             'fillOpacity',
             'dashArray',
             'popupTemplate',
-            'captionBar'
+            'zoomTo',
+            'captionBar',
+            'slideshow',
+            'sortKey'
         ], properties = {};
         for (var i = editableOptions.length - 1; i >= 0; i--) {
-            if (typeof this.options[editableOptions[i]] !== "undefined") {
+            if (typeof this.options[editableOptions[i]] !== 'undefined') {
                 properties[editableOptions[i]] = this.options[editableOptions[i]];
             }
         }
         var geojson = {
-            type: "Feature",
+            type: 'Feature',
             geometry: this.geometry(),
             properties: properties
         };
         this.backupOptions();
         var formData = new FormData();
-        formData.append("name", this.options.name);
-        formData.append("center", JSON.stringify(this.geometry()));
-        formData.append("settings", JSON.stringify(geojson));
+        formData.append('name', this.options.name);
+        formData.append('center', JSON.stringify(this.geometry()));
+        formData.append('settings', JSON.stringify(geojson));
         this.post(this.getSaveUrl(), {
             data: formData,
             callback: function (data) {
@@ -845,7 +872,7 @@ L.Storage.Map.include({
         /* Return a GeoJSON geometry Object */
         var latlng = this.latLng(this.options.center || this.getCenter());
         return {
-            type: "Point",
+            type: 'Point',
             coordinates: [
                 latlng.lng,
                 latlng.lat
@@ -854,8 +881,7 @@ L.Storage.Map.include({
     },
 
     defaultDataLayer: function () {
-        var self = this,
-            datalayer;
+        var datalayer;
         for (var i in this.datalayers) {
             if (this.datalayers.hasOwnProperty(i)) {
                 datalayer = this.datalayers[i];
@@ -886,8 +912,7 @@ L.Storage.Map.include({
 
     edit: function () {
         if(!this.editEnabled) return;
-        var self = this,
-            container = L.DomUtil.create('div'),
+        var container = L.DomUtil.create('div'),
             metadataFields = [
                 'options.name',
                 'options.description',
@@ -928,11 +953,13 @@ L.Storage.Map.include({
             'options.fillOpacity',
             'options.dashArray',
             'options.popupTemplate',
+            'options.zoomTo',
+            ['options.sortKey', {handler: 'Input', helpText: L._('Property to use for sorting features'), placeholder: L._('Default: name')}]
         ];
 
         builder = new L.S.FormBuilder(this, optionsFields, {
             callback: function (field) {
-                if (field !== "options.popupTemplate") {
+                if (field !== 'options.popupTemplate') {
                     this.eachDataLayer(function (datalayer) {
                         datalayer.redraw();
                     });
@@ -947,7 +974,7 @@ L.Storage.Map.include({
         }
         var tilelayerFields = [
             ['options.tilelayer.name', {handler: 'BlurInput', placeholder: L._('display name')}],
-            ['options.tilelayer.url_template', {handler: 'BlurInput', helpText: L._("Supported scheme") + ': http://{s}.domain.com/{z}/{x}/{y}.png', placeholder: 'url'}],
+            ['options.tilelayer.url_template', {handler: 'BlurInput', helpText: L._('Supported scheme') + ': http://{s}.domain.com/{z}/{x}/{y}.png', placeholder: 'url'}],
             ['options.tilelayer.maxZoom', {handler: 'BlurIntInput', placeholder: L._('max zoom')}],
             ['options.tilelayer.minZoom', {handler: 'BlurIntInput', placeholder: L._('min zoom')}],
             ['options.tilelayer.attribution', {handler: 'BlurInput', placeholder: L._('attribution')}],
@@ -975,9 +1002,24 @@ L.Storage.Map.include({
             callbackContext: this
         });
         limitBounds.appendChild(boundsBuilder.build());
+        var slideshow = L.DomUtil.createFieldset(container, L._('Slideshow'));
+        var slideshowFields = [
+            ['options.slideshow.delay', {handler: 'IntInput', placeholder: L._('Set a value for adding a slideshow'), helpText: L._('Delay between elements (in milliseconds)')}],
+            ['options.slideshow.autorun', {handler: 'CheckBox', helpText: L._('Autostart slideshow when map is loaded?')}]
+        ];
+        var slideshowHandler = function () {
+            this.slideshow.setOptions(this.options.slideshow);
+            this.initControls();
+        };
+        var slideshowBuilder = new L.S.FormBuilder(this, slideshowFields, {
+            callback: slideshowHandler,
+            callbackContext: this
+        });
+        slideshow.appendChild(slideshowBuilder.build());
+
         var setCurrentButton = L.DomUtil.add('a', '', limitBounds, L._('Use current bounds'));
-        setCurrentButton.href = "#";
-        L.DomEvent.on(setCurrentButton, 'click', function (e) {
+        setCurrentButton.href = '#';
+        L.DomEvent.on(setCurrentButton, 'click', function () {
             var bounds = this.getBounds();
             this.options.limitBounds.south = L.Util.formatNum(bounds.getSouth());
             this.options.limitBounds.west = L.Util.formatNum(bounds.getWest());
@@ -990,13 +1032,13 @@ L.Storage.Map.include({
 
         var advancedActions = L.DomUtil.createFieldset(container, L._('Advanced actions'));
         var del = L.DomUtil.create('a', 'storage-delete', advancedActions);
-        del.href = "#";
+        del.href = '#';
         del.innerHTML = L._('Delete');
         L.DomEvent
             .on(del, 'click', L.DomEvent.stop)
             .on(del, 'click', this.del, this);
         var clone = L.DomUtil.create('a', 'storage-clone', advancedActions);
-        clone.href = "#";
+        clone.href = '#';
         clone.innerHTML = L._('Clone this map');
         L.DomEvent
             .on(clone, 'click', L.DomEvent.stop)
@@ -1004,14 +1046,16 @@ L.Storage.Map.include({
         L.S.fire('ui:start', {data: {html: container}});
     },
 
-    enableEdit: function(e) {
-        L.DomUtil.addClass(document.body, "storage-edit-enabled");
+    enableEdit: function() {
+        L.DomUtil.addClass(document.body, 'storage-edit-enabled');
         this.editEnabled = true;
+        this.fire('edit:enabled');
     },
 
-    disableEdit: function(e) {
-        L.DomUtil.removeClass(document.body, "storage-edit-enabled");
+    disableEdit: function() {
+        L.DomUtil.removeClass(document.body, 'storage-edit-enabled');
         this.editEnabled = false;
+        this.fire('edit:disabled');
     },
 
     getDisplayName: function () {
@@ -1033,17 +1077,23 @@ L.Storage.Map.include({
         L.DomEvent.on(about, 'click', this.displayCaption, this);
         var browser = L.DomUtil.add('a', 'storage-open-browser-link', container, ' | ' + L._('Browse data'));
         browser.href = '#';
-        L.DomEvent.on(browser, 'click', this.openBrowser, this);
+        L.DomEvent.on(browser, 'click', L.DomEvent.stop)
+                  .on(browser, 'click', this.openBrowser, this);
         var setName = function () {
             name.innerHTML = this.getDisplayName();
         };
         L.bind(setName, this)();
         this.on('synced', L.bind(setName, this));
+        if (this.options.slideshow && this.options.slideshow.delay) {
+            this.onceDatalayersLoaded(function () {
+                container.appendChild(this.slideshow.renderToolbox());
+            });
+        }
     },
 
     initEditBar: function () {
         var container = L.DomUtil.create('div', 'storage-main-edit-toolbox', this._controlContainer),
-            title = L.DomUtil.add('h3', '', container, L._("Editing") + '&nbsp;'),
+            title = L.DomUtil.add('h3', '', container, L._('Editing') + '&nbsp;'),
             name = L.DomUtil.create('a', 'storage-click-to-edit', title),
             setName = function () {
                 name.innerHTML = this.getDisplayName();
@@ -1052,15 +1102,15 @@ L.Storage.Map.include({
         L.DomEvent.on(name, 'click', this.edit, this);
         this.on('synced', L.bind(setName, this));
         this.help.button(title, 'edit');
-        var save = L.DomUtil.create('a', "leaflet-control-edit-save button", container);
+        var save = L.DomUtil.create('a', 'leaflet-control-edit-save button', container);
         save.href = '#';
-        save.title = L._("Save current edits") + ' (Ctrl-S)';
+        save.title = L._('Save current edits') + ' (Ctrl-S)';
         save.innerHTML = L._('Save');
-        var cancel = L.DomUtil.create('a', "leaflet-control-edit-cancel button", container);
+        var cancel = L.DomUtil.create('a', 'leaflet-control-edit-cancel button', container);
         cancel.href = '#';
-        cancel.title = L._("Cancel edits");
+        cancel.title = L._('Cancel edits');
         cancel.innerHTML = L._('Cancel');
-        var disable = L.DomUtil.create('a', "leaflet-control-edit-disable", container);
+        var disable = L.DomUtil.create('a', 'leaflet-control-edit-disable', container);
         disable.href = '#';
         disable.title = disable.innerHTML = L._('Disable editing');
 
@@ -1082,7 +1132,7 @@ L.Storage.Map.include({
     },
 
     askForReset: function (e) {
-        if (!confirm(L._("Are you sure you want to cancel your changes?"))) return;
+        if (!confirm(L._('Are you sure you want to cancel your changes?'))) return;
         this.disableEdit(e);
         L.S.fire('ui:end');
         this.reset();
