@@ -8,9 +8,10 @@ from django.contrib import messages
 from django.contrib.auth import logout as do_logout
 from django.contrib.auth.models import User
 from django.core.signing import Signer, BadSignature
-from django.core.urlresolvers import reverse_lazy
+from django.core.urlresolvers import reverse_lazy, reverse
 from django.http import (HttpResponse, HttpResponseForbidden,
-                         HttpResponseRedirect, CompatibleStreamingHttpResponse)
+                         HttpResponseRedirect, CompatibleStreamingHttpResponse,
+                         HttpResponsePermanentRedirect)
 from django.shortcuts import get_object_or_404
 from django.template import RequestContext
 from django.template.loader import render_to_string
@@ -23,9 +24,10 @@ from django.views.generic.list import ListView
 from django.views.generic.base import TemplateView, RedirectView
 from django.views.generic.edit import CreateView, UpdateView, DeleteView
 from django.utils.http import http_date
+from django.middleware.gzip import re_accepts_gzip
 
 from .models import Map, DataLayer, TileLayer, Pictogram, Licence
-from .utils import get_uri_template
+from .utils import get_uri_template, gzip_file
 from .forms import (DataLayerForm, UpdateMapPermissionsForm, MapSettingsForm,
                     AnonymousMapPermissionsForm, DEFAULT_LATITUDE,
                     DEFAULT_LONGITUDE, FlatErrorList)
@@ -43,7 +45,9 @@ def _urls_for_js(urls=None):
         # prevent circular import
         from .urls import urlpatterns
         urls = [url.name for url in urlpatterns if getattr(url, 'name', None)]
-    return dict(zip(urls, [get_uri_template(url) for url in urls]))
+    urls = dict(zip(urls, [get_uri_template(url) for url in urls]))
+    urls.update(getattr(settings, 'LEAFLET_STORAGE_EXTRA_URLS', {}))
+    return urls
 
 
 def render_to_json(templates, response_kwargs, context, request):
@@ -91,10 +95,7 @@ class MapDetailMixin(object):
         properties = {}
         properties['datalayers'] = self.get_datalayers()
         properties['urls'] = _urls_for_js()
-        properties['urls'].update(getattr(settings, 'LEAFLET_STORAGE_EXTRA_URLS', {}))
         properties['tilelayers'] = self.get_tilelayers()
-        # properties['name'] = self.object.name
-        # properties['description'] = self.object.description
         if self.get_short_url():
             properties['shortUrl'] = self.get_short_url()
 
@@ -107,9 +108,7 @@ class MapDetailMixin(object):
             context['locale'] = locale
         properties['allowEdit'] = self.is_edit_allowed()
         properties["default_iconUrl"] = "%sstorage/src/img/marker.png" % settings.STATIC_URL
-        # properties['center'] = simplejson.loads(self.object.center.geojson)
         properties['storage_id'] = self.get_storage_id()
-        # properties['zoom'] = self.object.zoom
         properties['licences'] = dict((l.name, l.json) for l in Licence.objects.all())
         # if properties['locateOnLoad']:
         #     properties['locate'] = {
@@ -152,10 +151,17 @@ class MapView(MapDetailMixin, DetailView):
 
     def get(self, request, *args, **kwargs):
         self.object = self.get_object()
+        canonical = self.get_canonical_url()
+        if not request.path == canonical:
+            canonical = "?".join([canonical, request.META['QUERY_STRING']])
+            return HttpResponsePermanentRedirect(canonical)
         if not self.object.can_view(request):
             return HttpResponseForbidden('Forbidden')
         return super(MapView, self).get(request, *args, **kwargs)
 
+    def get_canonical_url(self):
+        return self.object.get_absolute_url()
+
     def get_datalayers(self):
         datalayers = DataLayer.objects.filter(map=self.object)  # TODO manage state
         return [l.metadata for l in datalayers]
@@ -184,7 +190,24 @@ class MapView(MapDetailMixin, DetailView):
         return shortUrl
 
     def get_geojson(self):
-        return self.object.settings
+        settings = self.object.settings
+        if not "properties" in settings:
+            settings['properties'] = {}
+        if self.object.owner:
+            settings['properties']['author'] = {
+                'name': self.object.owner.get_username(),
+                'link': reverse('user_maps', args=(self.object.owner.get_username(), ))
+            }
+        return settings
+
+
+class MapViewGeoJSON(MapView):
+
+    def get_canonical_url(self):
+        return reverse('map_geojson', args=(self.object.pk, ))
+
+    def render_to_response(self, context, *args, **kwargs):
+        return HttpResponse(context['map_settings'])
 
 
 class MapNew(MapDetailMixin, TemplateView):
@@ -270,8 +293,10 @@ class MapDelete(DeleteView):
 
     def delete(self, *args, **kwargs):
         self.object = self.get_object()
-        if not self.request.user == self.object.owner:
+        if self.object.owner and self.request.user != self.object.owner:
             return HttpResponseForbidden(_('Only its owner can delete the map.'))
+        if not self.object.owner and not self.object.is_anonymous_owner(self.request):
+            return HttpResponseForbidden('Forbidden.')
         self.object.delete()
         return simple_json_response(redirect="/")
 
@@ -361,13 +386,40 @@ class DataLayerView(BaseDetailView):
     def render_to_response(self, context, **response_kwargs):
         path = self.object.geojson.path
         statobj = os.stat(path)
-        # TODO IMS
-        response = CompatibleStreamingHttpResponse(
-            open(path, 'rb'),
-            content_type='application/json'
-        )
-        response["Last-Modified"] = http_date(statobj.st_mtime)
-        response['ETag'] = '"%s"' % hashlib.md5(response.content).hexdigest()
+        response = None
+        ext = '.gz'
+
+        ae = self.request.META.get('HTTP_ACCEPT_ENCODING', '')
+        if re_accepts_gzip.search(ae) and getattr(settings, 'LEAFLET_STORAGE_GZIP', True):
+            gzip_path = "{path}{ext}".format(path=path, ext=ext)
+            up_to_date = True
+            if not os.path.exists(gzip_path):
+                up_to_date = False
+            else:
+                gzip_statobj = os.stat(gzip_path)
+                if statobj.st_mtime > gzip_statobj.st_mtime:
+                    up_to_date = False
+            if not up_to_date:
+                gzip_file(path, gzip_path)
+            path = gzip_path
+
+        if getattr(settings, 'LEAFLET_STORAGE_ACCEL_REDIRECT', False):
+            response = HttpResponse()
+            response['X-Accel-Redirect'] = path
+        elif getattr(settings, 'LEAFLET_STORAGE_X_SEND_FILE', False):
+            response = HttpResponse()
+            response['X-Sendfile'] = path
+        else:
+            # TODO IMS
+            response = CompatibleStreamingHttpResponse(
+                open(path, 'rb'),
+                content_type='application/json'
+            )
+            response["Last-Modified"] = http_date(statobj.st_mtime)
+            response['ETag'] = '"%s"' % hashlib.md5(response.content).hexdigest()
+            response['Content-Length'] = str(len(response.content))
+            if path.endswith(ext):
+                response['Content-Encoding'] = 'gzip'
         return response
 
 
